COMMON SHARE1,nx1,nx2,nx3
COMMON SHARE2,x1,x2,x3,d0,d1,d2,d3,d4,d5,d6,d7
;------------------------------------------------------------------------------
; Function TEST_ISHDF: Tests that file is valid HDF4 SDS
;
FUNCTION TEST_ISHDF, filename
CATCH, err
IF (err EQ 0) THEN RETURN, HDF_ISHDF(filename) $
          ELSE RETURN, 0
END
;------------------------------------------------------------------------------
; Procedure READHDF: Reads HDF4 SDS files generated by zeus3d
;  These files contain v1,v2,v3,[b1,b2,b3,]d,e
;  This routine does not know the names of the variables read from
;  the file.  Thus, it reads data into a series of arrays named d0,d1,...,d7,
;  with the number of arrays actually used equal to the number of variables
;  in the HDF file.
;
;  The x1,x2, and x3-coordinates are also stored with each data block; they
;  are read from the first block.  The coordinates may have an arbitrary
;  number of elements, corresponding to nx1,nx2, and nx3.
;
pro readhdf, filename
COMMON SHARE1,nx1,nx2,nx3
COMMON SHARE2,x1,x2,x3,d0,d1,d2,d3,d4,d5,d6,d7
;
; Open file and initialize the SDS interface
;
IF NOT TEST_ISHDF(filename) THEN BEGIN
  PRINT, 'Invalid HDF file...'
  RETURN
  ENDIF ELSE $
  PRINT, 'Valid HDF file. Opening "' + filename + '"'
newfileID=HDF_SD_START(filename, /READ)
HDF_SD_FILEINFO, newfileID, ndatasets, attributes
;
; Read x3 (first data set)
;
thisSDS=HDF_SD_SELECT(newfileID, 0)
HDF_SD_GETINFO, thisSDS, NAME=thisSDSName, NATTS=numAttributes
index0=HDF_SD_NAMETOINDEX(newfileID, thisSDSName)
sdsID0=HDF_SD_SELECT(newfileID, index0)
HDF_SD_GETDATA, sdsID0, x3
nx3 = SIZE(x3,/N_ELEMENTS)
;
; Read x2 (next data set)
;
thisSDS=HDF_SD_SELECT(newfileID, 1)
HDF_SD_GETINFO, thisSDS, NAME=thisSDSName, NATTS=numAttributes
index0=HDF_SD_NAMETOINDEX(newfileID, thisSDSName)
sdsID0=HDF_SD_SELECT(newfileID, index0)
HDF_SD_GETDATA, sdsID0, x2
nx2 = SIZE(x2,/N_ELEMENTS)
;
; Read x1 (third data set)
;
thisSDS=HDF_SD_SELECT(newfileID, 2)
HDF_SD_GETINFO, thisSDS, NAME=thisSDSName, NATTS=numAttributes
index1=HDF_SD_NAMETOINDEX(newfileID, thisSDSName)
sdsID1=HDF_SD_SELECT(newfileID, index1)
HDF_SD_GETDATA, sdsID1, x1
nx1 = SIZE(x1,/N_ELEMENTS)
print,'This file contains',ndatasets/4,' datasets of size Nx=',nx1,' Ny=',nx2,' Nz=',nx3
;
; Read v1,v2, and v3 (always present in ZEUS3D hdf dumps), skipping the
; x,y,z stored with each variable.  Data are stored in d0,...,d2
;
thisSDS=HDF_SD_SELECT(newfileID, 3)
HDF_SD_GETINFO, thisSDS, NAME=thisSDSName, NATTS=numAttributes
index=HDF_SD_NAMETOINDEX(newfileID, thisSDSName)
sdsID=HDF_SD_SELECT(newfileID, index)
HDF_SD_GETDATA, sdsID, d0
;
thisSDS=HDF_SD_SELECT(newfileID, 7)
HDF_SD_GETINFO, thisSDS, NAME=thisSDSName, NATTS=numAttributes
index=HDF_SD_NAMETOINDEX(newfileID, thisSDSName)
sdsID=HDF_SD_SELECT(newfileID, index)
HDF_SD_GETDATA, sdsID, d1
;
thisSDS=HDF_SD_SELECT(newfileID, 11)
HDF_SD_GETINFO, thisSDS, NAME=thisSDSName, NATTS=numAttributes
index=HDF_SD_NAMETOINDEX(newfileID, thisSDSName)
sdsID=HDF_SD_SELECT(newfileID, index)
HDF_SD_GETDATA, sdsID, d2
;
; Keep reading datasets as long as there are any.  The maximum number a ZEUS2D
; HDF file can contain is 8 (ndatasets=32)
;
IF (ndatasets GT 12) THEN BEGIN
  thisSDS=HDF_SD_SELECT(newfileID, 15)
  HDF_SD_GETINFO, thisSDS, NAME=thisSDSName, NATTS=numAttributes
  index=HDF_SD_NAMETOINDEX(newfileID, thisSDSName)
  sdsID=HDF_SD_SELECT(newfileID, index)
  HDF_SD_GETDATA, sdsID, d3
ENDIF
IF (ndatasets GT 16) THEN BEGIN
  thisSDS=HDF_SD_SELECT(newfileID, 19)
  HDF_SD_GETINFO, thisSDS, NAME=thisSDSName, NATTS=numAttributes
  index=HDF_SD_NAMETOINDEX(newfileID, thisSDSName)
  sdsID=HDF_SD_SELECT(newfileID, index)
  HDF_SD_GETDATA, sdsID, d4
ENDIF
IF (ndatasets GT 20) THEN BEGIN
  thisSDS=HDF_SD_SELECT(newfileID, 23)
  HDF_SD_GETINFO, thisSDS, NAME=thisSDSName, NATTS=numAttributes
  index=HDF_SD_NAMETOINDEX(newfileID, thisSDSName)
  sdsID=HDF_SD_SELECT(newfileID, index)
  HDF_SD_GETDATA, sdsID, d5
ENDIF
IF (ndatasets GT 24) THEN BEGIN
  thisSDS=HDF_SD_SELECT(newfileID, 27)
  HDF_SD_GETINFO, thisSDS, NAME=thisSDSName, NATTS=numAttributes
  index=HDF_SD_NAMETOINDEX(newfileID, thisSDSName)
  sdsID=HDF_SD_SELECT(newfileID, index)
  HDF_SD_GETDATA, sdsID, d6
ENDIF
IF (ndatasets GT 28) THEN BEGIN
  thisSDS=HDF_SD_SELECT(newfileID, 31)
  HDF_SD_GETINFO, thisSDS, NAME=thisSDSName, NATTS=numAttributes
  index=HDF_SD_NAMETOINDEX(newfileID, thisSDSName)
  sdsID=HDF_SD_SELECT(newfileID, index)
  HDF_SD_GETDATA, sdsID, d7
ENDIF
;
;  Close HDF file and end
;
HDF_SD_END, newfileID
END
;------------------------------------------------------------------------------
; Procedure PLOT1 : Makes 1D plot of single variable using points
;
PRO plot1,coord,var

IF (SIZE(var,/N_DIMENSIONS) NE 1) THEN BEGIN
  PRINT, 'Variable is not a 1-D array...'
  RETURN
ENDIF
IF (SIZE(coord,/N_ELEMENTS) NE SIZE(var,/N_ELEMENTS)) THEN BEGIN
  PRINT, 'Coordinate and variable have different number of elements...'
  RETURN
ENDIF

ymin=MIN(var)
ymax=MAX(var)
plot,coord,var,psym=6,symsize=.4,YRANGE=[ymin,ymax]

END
;------------------------------------------------------------------------------
; Procedure PLOT2 : Makes 2D contour plot of single variable with 30 levels
;
PRO plot2,coord1,coord2,var

sizes = SIZE(var)
IF (sizes[0] NE 2) THEN BEGIN
  PRINT, 'Variable is not a 2-D array...'
  RETURN
ENDIF
IF (SIZE(coord1,/N_ELEMENTS) NE sizes[1]) THEN BEGIN
  PRINT, 'Coordinate1 and leading dimension of variable have different number of elements...'
  RETURN
ENDIF
IF (SIZE(coord2,/N_ELEMENTS) NE sizes[2]) THEN BEGIN
  PRINT, 'Coordinate2 and second dimension of variable have different number of elements...'
  RETURN
ENDIF

contour,var,coord1,coord2,NLEVELS=30

END
